@@ -1,7 +1,7 @@
 /*
  *  Simple MPI demonstration program
  *
- *  Copyright (C) 2006-2010, Brainspark B.V.
+ *  Copyright (C) 2006-2011, Brainspark B.V.
  *
  *  This file is part of PolarSSL (http://www.polarssl.org)
  *  Lead Maintainer: Paul Bakker <polarssl_maintainer at polarssl.org>
@@ -29,14 +29,29 @@
 
 #include <stdio.h>
 
+#include "polarssl/config.h"
 #include "polarssl/bignum.h"
 
-int main( void )
+#if !defined(POLARSSL_BIGNUM_C) || !defined(POLARSSL_FS_IO)
+int main( int argc, char *argv[] )
+{
+    ((void) argc);
+    ((void) argv);
+
+    printf("POLARSSL_BIGNUM_C and/or POLARSSL_FS_IO not defined.\n");
+    return( 0 );
+}
+#else
+int main( int argc, char *argv[] )
 {
     mpi E, P, Q, N, H, D, X, Y, Z;
 
-    mpi_init( &E, &P, &Q, &N, &H,
-              &D, &X, &Y, &Z, NULL );
+    ((void) argc);
+    ((void) argv);
+
+    mpi_init( &E ); mpi_init( &P ); mpi_init( &Q ); mpi_init( &N );
+    mpi_init( &H ); mpi_init( &D ); mpi_init( &X ); mpi_init( &Y );
+    mpi_init( &Z );
 
     mpi_read_string( &P, 10, "2789" );
     mpi_read_string( &Q, 10, "3203" );
@@ -51,6 +66,7 @@ int main( void )
     mpi_write_file( "  P = ", &P, 10, NULL );
     mpi_write_file( "  Q = ", &Q, 10, NULL );
 
+#if defined(POLARSSL_GENPRIME)
     mpi_sub_int( &P, &P, 1 );
     mpi_sub_int( &Q, &Q, 1 );
     mpi_mul_mpi( &H, &P, &Q );
@@ -58,7 +74,9 @@ int main( void )
 
     mpi_write_file( "  D = E^-1 mod (P-1)*(Q-1) = ",
                     &D, 10, NULL );
-
+#else
+    printf("\nTest skipped (POLARSSL_GENPRIME not defined).\n\n");
+#endif
     mpi_read_string( &X, 10, "55555" );
     mpi_exp_mod( &Y, &X, &E, &N, NULL );
     mpi_exp_mod( &Z, &Y, &D, &N, NULL );
@@ -69,13 +87,15 @@ int main( void )
     mpi_write_file( "  Z (decrypted)  = Y^D mod N = ", &Z, 10, NULL );
     printf( "\n" );
 
-    mpi_free( &Z, &Y, &X, &D, &H,
-              &N, &Q, &P, &E, NULL );
+    mpi_free( &E ); mpi_free( &P ); mpi_free( &Q ); mpi_free( &N );
+    mpi_free( &H ); mpi_free( &D ); mpi_free( &X ); mpi_free( &Y );
+    mpi_free( &Z );
 
-#ifdef WIN32
+#if defined(_WIN32)
     printf( "  Press Enter to exit this program.\n" );
     fflush( stdout ); getchar();
 #endif
 
     return( 0 );
 }
+#endif /* POLARSSL_BIGNUM_C && POLARSSL_FS_IO */
